Class: WaterDrop::Instrumentation::Notifications
- Inherits:
-
Karafka::Core::Monitoring::Notifications
- Object
- Karafka::Core::Monitoring::Notifications
- WaterDrop::Instrumentation::Notifications
- Defined in:
- lib/waterdrop/instrumentation/notifications.rb
Overview
Instrumented is used to hookup external monitoring services to monitor how WaterDrop works
Constant Summary collapse
- EVENTS =
Note:
The non-error once support timestamp benchmarking
List of events that we support in the system and to which a monitor client can hook up
%w[ producer.closed message.produced_async message.produced_sync message.acknowledged message.buffered messages.produced_async messages.produced_sync messages.buffered buffer.flushed_async buffer.flushed_sync statistics.emitted error.occurred ].freeze
Instance Method Summary collapse
-
#initialize ⇒ WaterDrop::Instrumentation::Monitor
constructor
Monitor instance for system instrumentation.
Constructor Details
#initialize ⇒ WaterDrop::Instrumentation::Monitor
Returns monitor instance for system instrumentation.
30 31 32 33 |
# File 'lib/waterdrop/instrumentation/notifications.rb', line 30 def initialize super EVENTS.each { |event| register_event(event) } end |