Module: WaterDrop::Producer::Async
- Included in:
- WaterDrop::Producer
- Defined in:
- lib/waterdrop/producer/async.rb
Overview
Component for asynchronous producer operations
Instance Method Summary collapse
-
#produce_async(message) ⇒ Rdkafka::Producer::DeliveryHandle
Produces a message to Kafka and does not wait for results.
-
#produce_many_async(messages) ⇒ Array<Rdkafka::Producer::DeliveryHandle>
Produces many messages to Kafka and does not wait for them to be delivered.
Instance Method Details
#produce_async(message) ⇒ Rdkafka::Producer::DeliveryHandle
Produces a message to Kafka and does not wait for results
16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 |
# File 'lib/waterdrop/producer/async.rb', line 16 def produce_async() ensure_active! = middleware.run() () @monitor.instrument( 'message.produced_async', producer_id: id, message: ) { produce() } rescue *SUPPORTED_FLOW_ERRORS re_raised = Errors::ProduceError.new @monitor.instrument( 'error.occurred', producer_id: id, message: , error: re_raised, type: 'message.produce_async' ) raise re_raised end |
#produce_many_async(messages) ⇒ Array<Rdkafka::Producer::DeliveryHandle>
Produces many messages to Kafka and does not wait for them to be delivered
51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 |
# File 'lib/waterdrop/producer/async.rb', line 51 def produce_many_async() ensure_active! dispatched = [] = middleware.run_many() .each { || () } @monitor.instrument( 'messages.produced_async', producer_id: id, messages: ) do .each do || dispatched << produce() end dispatched end rescue *SUPPORTED_FLOW_ERRORS re_raised = Errors::ProduceManyError.new(dispatched) @monitor.instrument( 'error.occurred', producer_id: id, messages: , dispatched: dispatched, error: re_raised, type: 'messages.produce_many_async' ) raise re_raised end |