Class: WaterDrop::Contracts::Message

Inherits:
Base
  • Object
show all
Defined in:
lib/waterdrop/contracts/message.rb

Overview

Contract with validation rules for validating that all the message options that we provide to producer ale valid and usable

Method Summary

Methods inherited from Base

#validate!