Class: WaterDrop::Contracts::Message
- Defined in:
- lib/waterdrop/contracts/message.rb
Overview
Contract with validation rules for validating that all the message options that we provide to producer ale valid and usable
Contract with validation rules for validating that all the message options that we provide to producer ale valid and usable