Module: WaterDrop::Patches::Rdkafka::Producer
- Defined in:
- lib/waterdrop/patches/rdkafka/producer.rb
Overview
Rdkafka::Producer patches
Instance Method Summary collapse
-
#name ⇒ String
Adds a method that allows us to get the native kafka producer name.
Instance Method Details
#name ⇒ String
Adds a method that allows us to get the native kafka producer name
12 13 14 |
# File 'lib/waterdrop/patches/rdkafka/producer.rb', line 12 def name ::Rdkafka::Bindings.rd_kafka_name(@native_kafka) end |